@@ -7,11 +7,12 @@ import matplotlib.pyplot as plt
 import os.path
 
 try:
-  import gi
-  gi.require_version('NumCosmo', '1.0')
-  gi.require_version('NumCosmoMath', '1.0')
+    import gi
+
+    gi.require_version('NumCosmo', '1.0')
+    gi.require_version('NumCosmoMath', '1.0')
 except:
-  pass
+    pass
 
 from gi.repository import GObject
 from gi.repository import NumCosmo as Nc
@@ -26,24 +27,24 @@ from py_sline_mfunc import PyTestFunc
 #  Initializing the library objects, this must be called before
 #  any other library function.
 #
-Ncm.cfg_init ()
+Ncm.cfg_init()
 
 #
 # Instantiating a new SLine model object and setting
 # some values for its parameters.
 #
-slm = PySLineModel ()
+slm = PySLineModel()
 slm.props.alpha = 0.9
-slm.props.a     = 0.2
+slm.props.a = 0.2
 
 #
 # New Model set object including slm with parameters
 # set as free.
 #
-mset = Ncm.MSet.empty_new ()
-mset.set (slm)
-mset.param_set_all_ftype (Ncm.ParamType.FREE)
-mset.prepare_fparam_map ()
+mset = Ncm.MSet.empty_new()
+mset.set(slm)
+mset.param_set_all_ftype(Ncm.ParamType.FREE)
+mset.prepare_fparam_map()
 
 #
 # Creating a new Serialization object, and load
@@ -51,41 +52,41 @@ mset.prepare_fparam_map ()
 #
 sld = None
 data_file = "example_data.obj"
-ser = Ncm.Serialize.new (0)
-if not os.path.exists (data_file):
-  print ("data file does not exists, run example_create_data.py first.")
-  sys.exit (-1)
+ser = Ncm.Serialize.new(0)
+if not os.path.exists(data_file):
+    print("data file does not exists, run example_create_data.py first.")
+    sys.exit(-1)
 else:
-  sld = ser.from_binfile (data_file)
+    sld = ser.from_binfile(data_file)
 
 #
 # New data set object with sld added.
 #
-dset = Ncm.Dataset.new ()
-dset.append_data (sld)
+dset = Ncm.Dataset.new()
+dset.append_data(sld)
 
 #
 # New likelihood object using dset.
 #
-lh = Ncm.Likelihood.new (dset)
+lh = Ncm.Likelihood.new(dset)
 
 #
 #  Creating a Fit object of type NLOPT using the fitting algorithm ln-neldermead to
 #  fit the Modelset mset using the Likelihood lh and using a numerical differentiation
 #  algorithm (NUMDIFF_FORWARD) to obtain the gradient (if needed).
 #
-fit = Ncm.Fit.new (Ncm.FitType.NLOPT, "ln-neldermead", lh, mset, Ncm.FitGradType.NUMDIFF_FORWARD)
+fit = Ncm.Fit.new(Ncm.FitType.NLOPT, "ln-neldermead", lh, mset, Ncm.FitGradType.NUMDIFF_FORWARD)
 
 #
 # Printing fitting informations.
 #
-fit.log_info ()
+fit.log_info()
 
 #
 # Setting single thread calculation.
 #
-Ncm.func_eval_set_max_threads (0)
-Ncm.func_eval_log_pool_stats ()
+Ncm.func_eval_set_max_threads(0)
+Ncm.func_eval_log_pool_stats()
 
 #
 # New Gaussian prior to provide the initial points for the chain.
@@ -95,10 +96,10 @@ Ncm.func_eval_log_pool_stats ()
 # The initial sampler will use a diagonal covariance with the
 # diagonal terms being the parameters scale set by each model.
 #
-init_sampler = Ncm.MSetTransKernGauss.new (0)
-init_sampler.set_mset (mset)
-init_sampler.set_prior_from_mset ()
-init_sampler.set_cov_from_rescale (1.0)
+init_sampler = Ncm.MSetTransKernGauss.new(0)
+init_sampler.set_mset(mset)
+init_sampler.set_prior_from_mset()
+init_sampler.set_cov_from_rescale(1.0)
 
 #
 # Creates the ESMCMC walker object, this object is responsible
@@ -106,25 +107,25 @@ init_sampler.set_cov_from_rescale (1.0)
 # is affine invariant and therefore gives good results even for
 # very correlated parametric space.
 # 
-nwalkers = 20
-stretch = Ncm.FitESMCMCWalkerStretch.new (nwalkers, mset.fparams_len ())
+nwalkers = 300
+apes = Ncm.FitESMCMCWalkerAPES.new(nwalkers, mset.fparams_len())
 
 #
 # The methods below set the walk scale, which controls the size of the
 # step done between two walkers and circumscribe the walkers inside
 # the box defined by the parameters inside the mset object.
 #
-stretch.set_scale (3.0)
-stretch.set_box_mset (mset)
+#stretch.set_scale(3.0)
+#stretch.set_box_mset(mset)
 
 #
 # Additional functions of mset to be computed during the sampling 
 # process.
 #
-mfunc_oa = Ncm.ObjArray.new ()
+mfunc_oa = Ncm.ObjArray.new()
 
-tf = PyTestFunc ()
-mfunc_oa.add (tf)
+tf = PyTestFunc()
+mfunc_oa.add(tf)
 
 #
 # Initialize the ESMCMC object using the objects above. It will
@@ -133,7 +134,7 @@ mfunc_oa.add (tf)
 # in the chain (the last 50 parametric points) to calculate the
 # proposal points.
 #
-esmcmc  = Ncm.FitESMCMC.new_funcs_array (fit, nwalkers, init_sampler, stretch, Ncm.FitRunMsgs.SIMPLE, mfunc_oa)
+esmcmc = Ncm.FitESMCMC.new_funcs_array(fit, nwalkers, init_sampler, apes, Ncm.FitRunMsgs.SIMPLE, mfunc_oa)
 
 #
 # These methods enable the auto-trim options on ESMCMC. This option 
@@ -142,33 +143,33 @@ esmcmc  = Ncm.FitESMCMC.new_funcs_array (fit, nwalkers, init_sampler, stretch, N
 # chains in blocks of n/100. The last method asserts that each 2min
 # the catalog will be checked.
 #
-esmcmc.set_auto_trim (True)
-esmcmc.set_auto_trim_div (100)
-esmcmc.set_max_runs_time (2.0 * 60.0)
-esmcmc.set_nthreads (2)
+# esmcmc.set_auto_trim(True)
+# esmcmc.set_auto_trim_div(100)
+esmcmc.set_max_runs_time(2.0 * 60.0)
+esmcmc.set_nthreads(2)
 
 #
 # Using `example_esmcmc_out.fits' as the catalog file, if there
 # is already data in it, the sampler continues from where it stopped.
 # 
-esmcmc.set_data_file ("example_esmcmc_out.fits")
+esmcmc.set_data_file("example_esmcmc_out.fits")
 
 #
 # Running the esmcmc, it will first calculate 1000 points, after that
 # it will estimate the error in the parameters mean. Using the current
-# errors the algorithm tries to calculated how many extra steps are 
-# necessary to obtain the required error `10^-3' in every parameters,
+# errors the algorithm tries to calculate how many extra steps are
+# necessary to obtain the required error `10^-3' in every parameter,
 # and it will run such extra steps. It will repeat this procedure
 # until it attains the required error in every parameter.
 # 
 #
-esmcmc.start_run ()
-esmcmc.run_lre (10, 1.0e-3)
-esmcmc.end_run ()
+esmcmc.start_run()
+esmcmc.run_lre(10, 1.0e-3)
+esmcmc.end_run()
 
 #
 # Calculates the parameter means and covariance and set it into 
 # the fit object and then print.
 # 
-esmcmc.mean_covar ()
-fit.log_covar ()
+esmcmc.mean_covar()
+fit.log_covar()
