@@ -146,11 +146,20 @@ struct spectra {
   int ln_k_size;    /**< number ln(k) values */
   double * ln_k;    /**< list of ln(k) values ln_k[index_k] */
 
-  int ln_tau_size;  /**< number ln(tau) values (only one if z_max_pk = 0) */
-  double * ln_tau;  /**< list of ln(tau) values ln_tau[index_tau] */
+  int ln_tau_size;  /**< number of ln(tau) values, for the matter
+                       power spectrum and the matter transfer
+                       functions, (only one if z_max_pk = 0) */
+
+  double * ln_tau;  /**< list of ln(tau) values ln_tau[index_tau], for
+                       the matter power spectrum and the matter
+                       transfer functions, in growing order. So
+                       exp(ln_tau[0]) is the earliest time
+                       (i.e. highest redshift), while
+                       exp(ln_tau[ln_tau_size-1]) is today (i.e
+                       z=0). */
 
   double * ln_pk;   /**< Matter power spectrum.
-                       depends on indices index_md, index_ic1, index_ic2, index_k, index_tau as:
+                       depends on indices index_ic1_ic2, index_k, index_tau as:
                        ln_pk[(index_tau * psp->k_size + index_k)* psp->ic_ic_size[index_md] + index_ic1_ic2]
                        where index_ic1_ic2 labels ordered pairs (index_ic1, index_ic2) (since
                        the primordial spectrum is symmetric in (index_ic1, index_ic2)).
@@ -176,12 +185,46 @@ struct spectra {
 
   double sigma8;    /**< sigma8 parameter */
 
+  double sigma8_cb; /**< if ncdm present: contribution to sigma8 from only baryons and cdm */
+
+  double * ln_pk_l;   /**q< Total linear matter power spectrum, just
+                           depending on indices index_k, index_tau as:
+                           ln_pk[index_tau * psp->k_size + index_k]
+                           Range of k and tau value identical to
+                           ln_pk array. */
+
+  double * ddln_pk_l; /**< second derivative of above array with respect to log(tau), for spline interpolation. */
+
+  int ln_tau_nl_size;  /**< number of ln(tau) values for non-linear
+                          spectrum (possibly smaller than ln_tau_size,
+                          because the non-linear spectrum is stored
+                          only in the time/redhsift range where the
+                          non-linear corrections were really computed,
+                          to avoid dealing with discontinuities in
+                          the spline interpolation) */
+
+  double * ln_tau_nl;  /**< list of ln(tau) values
+                          ln_tau_nl[index_tau], for the non-linear
+                          power spectrum, in growing order. So
+                          exp(ln_tau_nl[0]) is the earliest time
+                          (i.e. highest redshift), while
+                          exp(ln_tau_nl[ln_tau_nl_size-1]) is today
+                          (i.e z=0). */
+
   double * ln_pk_nl;   /**< Non-linear matter power spectrum.
                           depends on indices index_k, index_tau as:
-                          ln_pk_nl[index_tau * psp->k_size + index_k]
-                    */
+                          ln_pk_nl[index_tau * psp->k_size + index_k] */
   double * ddln_pk_nl; /**< second derivative of above array with respect to log(tau), for spline interpolation. */
 
+  double * ln_pk_cb;           /**< same as ln_pk for baryon+cdm component only */
+  double * ddln_pk_cb;         /**< same as ddln_pk for baryon+cdm component only */
+
+  double * ln_pk_cb_l;         /**< same as ln_pk_l for baryon+cdm component only */
+  double * ddln_pk_cb_l;       /**< same as ddln_pk_l for baryon+cdm component only */
+
+  double * ln_pk_cb_nl;        /**< same as ln_pk_nl for baryon+cdm component only */
+  double * ddln_pk_cb_nl;      /**< same as ddln_pk_nl for baryon+cdm component only */
+
   int index_tr_delta_g;        /**< index of gamma density transfer function */
   int index_tr_delta_b;        /**< index of baryon density transfer function */
   int index_tr_delta_cdm;      /**< index of cold dark matter density transfer function */
@@ -204,6 +247,11 @@ struct spectra {
   int index_tr_theta_tot;      /**< index of total matter velocity transfer function */
   int index_tr_phi;            /**< index of Bardeen potential phi */
   int index_tr_psi;            /**< index of Bardeen potential psi */
+  int index_tr_phi_prime;      /**< index of derivative of Bardeen potential phi */
+  int index_tr_h;              /**< index of synchronous gauge metric perturbation h */
+  int index_tr_h_prime;        /**< index of synchronous gauge metric perturbation h' */
+  int index_tr_eta;            /**< index of synchronous gauge metric perturbation eta */
+  int index_tr_eta_prime;      /**< index of synchronous gauge metric perturbation eta' */
   int tr_size;                 /**< total number of species in transfer functions */
 
   double * matter_transfer;   /**< Matter transfer functions.
@@ -261,7 +309,9 @@ extern "C" {
                       enum linear_or_logarithmic mode,
                       double z,
                       double * output_tot,
-                      double * output_ic
+                      double * output_ic,
+                      double * output_cb_tot,
+                      double * output_cb_ic
                       );
 
   int spectra_pk_at_k_and_z(
@@ -271,7 +321,9 @@ extern "C" {
                             double k,
                             double z,
                             double * pk,
-                            double * pk_ic
+                            double * pk_ic,
+                            double * pk_cb,
+                            double * pk_cb_ic
                             );
 
   int spectra_pk_nl_at_z(
@@ -279,7 +331,8 @@ extern "C" {
                          struct spectra * psp,
                          enum linear_or_logarithmic mode,
                          double z,
-                         double * output_tot
+                         double * output_tot,
+                         double * output_cb_tot
                          );
 
   int spectra_pk_nl_at_k_and_z(
@@ -288,7 +341,8 @@ extern "C" {
                                struct spectra * psp,
                                double k,
                                double z,
-                               double * pk_tot
+                               double * pk_tot,
+                               double * pk_cb_tot
                                );
 
   int spectra_tk_at_z(
@@ -356,6 +410,7 @@ extern "C" {
   int spectra_k_and_tau(
                         struct background * pba,
                         struct perturbs * ppt,
+                        struct nonlinear *pnl,
                         struct spectra * psp
                         );
 
@@ -376,6 +431,15 @@ extern "C" {
                     double *sigma
                     );
 
+  int spectra_sigma_cb(
+                    struct background * pba,
+                    struct primordial * ppm,
+                    struct spectra * psp,
+                    double R,
+                    double z,
+                    double *sigma_cb
+                    );
+
   int spectra_matter_transfers(
                                struct background * pba,
                                struct perturbs * ppt,
@@ -403,6 +467,17 @@ extern "C" {
                                      char first_line[_LINE_LENGTH_MAX_],
                                      FileName ic_suffix);
 
+  int spectra_fast_pk_at_kvec_and_zvec(
+				       struct background * pba,
+				       struct spectra * psp,
+				       double * kvec,
+				       int kvec_size,
+				       double * zvec,
+				       int zvec_size,
+				       double * pk_tot_out, /* (must be already allocated with kvec_size*zvec_size) */
+                       double * pk_cb_tot_out,
+				       int nonlinear);
+
 #ifdef __cplusplus
 }
 #endif
