@@ -88,7 +88,7 @@ int lensing_init(
                  struct nonlinear * pnl,
                  struct lensing * ple
                  ) {
-  
+
   /** Summary: */
   /** - Define local variables */
 
@@ -97,25 +97,25 @@ int lensing_init(
   double * w8; /* Corresponding Gauss-Legendre quadrature weights */
   double theta,delta_theta;
 
-  double ** d00  = NULL;  /* dmn[index_mu][index_l] */
-  double ** d11  = NULL;
-  double ** d2m2 = NULL;
-  double ** d22  = NULL;
-  double ** d20  = NULL;
-  double ** d1m1 = NULL;
-  double ** d31  = NULL;
-  double ** d40  = NULL;
+  double ** d00;  /* dmn[index_mu][index_l] */
+  double ** d11;
+  double ** d2m2;
+  double ** d22 = NULL;
+  double ** d20 = NULL;
+  double ** d1m1;
+  double ** d31 = NULL;
+  double ** d40 = NULL;
   double ** d3m1 = NULL;
   double ** d3m3 = NULL;
   double ** d4m2 = NULL;
   double ** d4m4 = NULL;
-  double * buf_dxx = NULL; /* buffer */
+  double * buf_dxx; /* buffer */
 
   double * Cgl;   /* Cgl[index_mu] */
   double * Cgl2;  /* Cgl2[index_mu] */
   double * sigma2; /* sigma[index_mu] */
 
-  double * ksi  = NULL;  /* ksi[index_mu] */
+  double * ksi = NULL;  /* ksi[index_mu] */
   double * ksiX = NULL;  /* ksiX[index_mu] */
   double * ksip = NULL;  /* ksip[index_mu] */
   double * ksim = NULL;  /* ksim[index_mu] */
@@ -133,12 +133,12 @@ int lensing_init(
   int num_mu,index_mu,icount;
   int l;
   double ll;
-  double * cl_unlensed = NULL;  /* cl_unlensed[index_ct] */
-  double * cl_tt = NULL; /* unlensed  cl, to be filled to avoid repeated calls to spectra_cl_at_l */
+  double * cl_unlensed;  /* cl_unlensed[index_ct] */
+  double * cl_tt; /* unlensed  cl, to be filled to avoid repeated calls to spectra_cl_at_l */
   double * cl_te = NULL; /* unlensed  cl, to be filled to avoid repeated calls to spectra_cl_at_l */
   double * cl_ee = NULL; /* unlensed  cl, to be filled to avoid repeated calls to spectra_cl_at_l */
   double * cl_bb = NULL; /* unlensed  cl, to be filled to avoid repeated calls to spectra_cl_at_l */
-  double * cl_pp = NULL; /* potential cl, to be filled to avoid repeated calls to spectra_cl_at_l */
+  double * cl_pp; /* potential cl, to be filled to avoid repeated calls to spectra_cl_at_l */
 
   double res,resX,lens;
   double resp, resm, lensp, lensm;
