@@ -8,17 +8,17 @@
 /*
  * numcosmo
  * Copyright (C) Mariana Penna Lima 2012 <pennalima@gmail.com>
- * 
+ *
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
- * 
+ *
  * numcosmo is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -37,7 +37,7 @@
 #include "build_cfg.h"
 
 #include "lss/nc_cluster_mass_benson.h"
-#include "math/integral.h"
+#include "math/ncm_integrate.h"
 #include "math/ncm_memory_pool.h"
 #include "math/ncm_cfg.h"
 
@@ -68,16 +68,17 @@ nc_cluster_mass_benson_init (NcClusterMassBenson *msz)
 {
   msz->signif_obs_min = 0.0;
   msz->signif_obs_max = 0.0;
-  msz->z0 = 0.0;
-  msz->M0 = 0.0;
+  msz->z0             = 0.0;
+  msz->M0             = 0.0;
 }
 
 static void
-_nc_cluster_mass_benson_set_property (GObject * object, guint prop_id, const GValue * value, GParamSpec * pspec)
+_nc_cluster_mass_benson_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
   NcClusterMassBenson *msz = NC_CLUSTER_MASS_BENSON (object);
+  
   g_return_if_fail (NC_IS_CLUSTER_MASS_BENSON (object));
-
+  
   switch (prop_id)
   {
     case PROP_SIGNIFICANCE_OBS_MIN:
@@ -102,8 +103,9 @@ static void
 _nc_cluster_mass_benson_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
   NcClusterMassBenson *msz = NC_CLUSTER_MASS_BENSON (object);
+  
   g_return_if_fail (NC_IS_CLUSTER_MASS_BENSON (object));
-
+  
   switch (prop_id)
   {
     case PROP_SIGNIFICANCE_OBS_MIN:
@@ -127,44 +129,30 @@ _nc_cluster_mass_benson_get_property (GObject *object, guint prop_id, GValue *va
 static void
 _nc_cluster_mass_benson_finalize (GObject *object)
 {
-
   /* Chain up : end */
   G_OBJECT_CLASS (nc_cluster_mass_benson_parent_class)->finalize (object);
 }
 
-guint _nc_cluster_mass_benson_obs_len (NcClusterMass *clusterm) { NCM_UNUSED (clusterm); return 1; }
-guint _nc_cluster_mass_benson_obs_params_len (NcClusterMass *clusterm) { NCM_UNUSED (clusterm); return 0; }
 static gdouble _nc_cluster_mass_benson_significance_m_p (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, const gdouble *xi, const gdouble *xi_params);
 static gdouble _nc_cluster_mass_benson_intp (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z);
 static void _nc_cluster_mass_benson_p_limits (NcClusterMass *clusterm, NcHICosmo *model, const gdouble *xi, const gdouble *xi_params, gdouble *lnM_lower, gdouble *lnM_upper);
 static void _nc_cluster_mass_benson_n_limits (NcClusterMass *clusterm, NcHICosmo *model, gdouble *lnM_lower, gdouble *lnM_upper);
 static gboolean _nc_cluster_mass_benson_resample (NcClusterMass *clusterm, NcHICosmo *model, gdouble lnM, gdouble z, gdouble *xi, const gdouble *xi_params, NcmRNG *rng);
 
-
 static void
 nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
 {
-  GObjectClass* object_class       = G_OBJECT_CLASS (klass);
-  NcClusterMassClass* parent_class = NC_CLUSTER_MASS_CLASS (klass);
+  GObjectClass *object_class       = G_OBJECT_CLASS (klass);
+  NcClusterMassClass *parent_class = NC_CLUSTER_MASS_CLASS (klass);
   NcmModelClass *model_class       = NCM_MODEL_CLASS (klass);
 
-  parent_class->P = &_nc_cluster_mass_benson_significance_m_p;
-  parent_class->intP = &_nc_cluster_mass_benson_intp;
-  parent_class->P_limits = &_nc_cluster_mass_benson_p_limits;
-  parent_class->N_limits = &_nc_cluster_mass_benson_n_limits;
-  parent_class->resample = &_nc_cluster_mass_benson_resample;
-  parent_class->obs_len = &_nc_cluster_mass_benson_obs_len;
-  parent_class->obs_params_len = &_nc_cluster_mass_benson_obs_params_len;
-
-  ncm_model_class_add_impl_flag (model_class, NC_CLUSTER_MASS_IMPL_ALL);
-
   model_class->set_property = &_nc_cluster_mass_benson_set_property;
   model_class->get_property = &_nc_cluster_mass_benson_get_property;
-  object_class->finalize = _nc_cluster_mass_benson_finalize;
-
+  object_class->finalize    = &_nc_cluster_mass_benson_finalize;
+  
   ncm_model_class_set_name_nick (model_class, "Benson - SZ", "Benson_SZ");
   ncm_model_class_add_params (model_class, 4, 0, PROP_SIZE);
-
+  
   /**
    * NcClusterMassBenson:signif_obs_min:
    *
@@ -176,7 +164,8 @@ nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
                                                         NULL,
                                                         "Minimum obsevational significance",
                                                         2.0, G_MAXDOUBLE, 5.0,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  
   /**
    * NcClusterMassBenson:signif_obs_max:
    *
@@ -188,7 +177,8 @@ nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
                                                         NULL,
                                                         "Maximum obsevational significance",
                                                         2.0, G_MAXDOUBLE, 40.0,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  
   /**
    * NcClusterMassBenson:z0:
    *
@@ -201,7 +191,8 @@ nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
                                                         NULL,
                                                         "Reference redshift",
                                                         0.0, G_MAXDOUBLE, 0.6,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB)); 
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  
   /**
    * NcClusterMassBenson:M0:
    *
@@ -214,10 +205,11 @@ nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
                                                         NULL,
                                                         "Reference mass",
                                                         1.0e13, G_MAXDOUBLE, 3.0e14,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
+  
   /**
    * NcClusterMassBenson:Asz:
-   * 
+   *
    * Slope of the SZ signal-mass scaling relation.
    * FIXME Set correct values (limits)
    */
@@ -225,10 +217,10 @@ nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
                               1e-8,  10.0, 1.0e-2,
                               NC_CLUSTER_MASS_BENSON_DEFAULT_PARAMS_ABSTOL, NC_CLUSTER_MASS_BENSON_DEFAULT_A_SZ,
                               NCM_PARAM_TYPE_FIXED);
-
+  
   /**
    * NcClusterMassBenson:Bsz:
-   * 
+   *
    * SZ signal-mass scaling parameter.
    * FIXME Set correct values (limits)
    */
@@ -236,10 +228,10 @@ nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
                               1e-8,  10.0, 1.0e-2,
                               NC_CLUSTER_MASS_BENSON_DEFAULT_PARAMS_ABSTOL, NC_CLUSTER_MASS_BENSON_DEFAULT_B_SZ,
                               NCM_PARAM_TYPE_FIXED);
-
+  
   /**
    * NcClusterMassBenson:Csz:
-   * 
+   *
    * SZ signal-mass scaling parameter.
    * FIXME Set correct values (limits)
    */
@@ -247,10 +239,10 @@ nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
                               1e-8,  10.0, 1.0e-2,
                               NC_CLUSTER_MASS_BENSON_DEFAULT_PARAMS_ABSTOL, NC_CLUSTER_MASS_BENSON_DEFAULT_C_SZ,
                               NCM_PARAM_TYPE_FIXED);
-
+  
   /**
    * NcClusterMassBenson:Dsz:
-   * 
+   *
    * Standard deviation of the SZ signal-mass scaling relation.
    * $D_sz \in [0.01, 2.0]$.
    */
@@ -258,11 +250,20 @@ nc_cluster_mass_benson_class_init (NcClusterMassBensonClass *klass)
                               1e-2,  2.0, 1.0e-2,
                               NC_CLUSTER_MASS_BENSON_DEFAULT_PARAMS_ABSTOL, NC_CLUSTER_MASS_BENSON_DEFAULT_D_SZ,
                               NCM_PARAM_TYPE_FIXED);
-
+  
   /* Check for errors in parameters initialization */
   ncm_model_class_check_params_info (model_class);
-}
 
+  parent_class->P              = &_nc_cluster_mass_benson_significance_m_p;
+  parent_class->intP           = &_nc_cluster_mass_benson_intp;
+  parent_class->P_limits       = &_nc_cluster_mass_benson_p_limits;
+  parent_class->N_limits       = &_nc_cluster_mass_benson_n_limits;
+  parent_class->resample       = &_nc_cluster_mass_benson_resample;
+  parent_class->obs_len        = 1;
+  parent_class->obs_params_len = 0;
+
+  ncm_model_class_add_impl_flag (model_class, NC_CLUSTER_MASS_IMPL_ALL);
+}
 
 typedef struct _integrand_data
 {
@@ -274,7 +275,7 @@ typedef struct _integrand_data
   const gdouble *xi;
   gdouble lnA;
   gdouble lnM0;
-  gdouble lnE_E0;  /* ln (E/E0)  */
+  gdouble lnE_E0; /* ln (E/E0)  */
   gdouble mu;
   gdouble D2_2;
 } integrand_data;
@@ -283,15 +284,15 @@ static gdouble
 _nc_cluster_mass_benson_xi_mean (gdouble zeta)
 {
   const gdouble xi_mean = (zeta <= 1.000001) ? zeta : sqrt (zeta * zeta + 3.0);
-
-  return xi_mean; 
+  
+  return xi_mean;
 }
 
 static gdouble
 _nc_cluster_mass_benson_int_xi_cut_inf (gdouble xi_mean, gdouble xi_cut)
 {
   const gdouble a = (xi_cut - xi_mean) / M_SQRT2;
-
+  
   if (a < 0.0)
     return (1.0 - erf (a)) * 0.5;
   else
@@ -301,19 +302,22 @@ _nc_cluster_mass_benson_int_xi_cut_inf (gdouble xi_mean, gdouble xi_cut)
 static gdouble
 _nc_cluster_mass_benson_significance_m_p_integrand (gdouble zeta, gpointer userdata)
 {
-  integrand_data *data = (integrand_data *) userdata;
+  integrand_data *data     = (integrand_data *) userdata;
   NcClusterMassBenson *msz = data->msz;
-  const gdouble lnzeta = log (zeta);
-  const gdouble xi_mean = _nc_cluster_mass_benson_xi_mean (zeta);
-  const gdouble y = data->xi[0] - xi_mean;
-  const gdouble x = lnzeta - data->mu;
-  const gdouble exp_arg = - y * y / 2.0 - x * x / data->D2_2;
-
+  const gdouble lnzeta     = log (zeta);
+  const gdouble xi_mean    = _nc_cluster_mass_benson_xi_mean (zeta);
+  const gdouble y          = data->xi[0] - xi_mean;
+  const gdouble x          = lnzeta - data->mu;
+  const gdouble exp_arg    = -y * y / 2.0 - x * x / data->D2_2;
+  
   if (exp_arg < GSL_LOG_DBL_MIN)
+  {
     return 0.0;
+  }
   else
   {
     const gdouble result = exp (exp_arg) / (2.0 * M_PI * D_SZ * zeta);
+    
     return result;
   }
 }
@@ -321,17 +325,17 @@ _nc_cluster_mass_benson_significance_m_p_integrand (gdouble zeta, gpointer userd
 static gdouble
 _nc_cluster_mass_benson_significance_m_intp_integrand (gdouble zeta, gpointer userdata)
 {
-  integrand_data *data = (integrand_data *) userdata;
-  NcClusterMassBenson *msz = data->msz;
-  const gdouble lnzeta = log (zeta);
-  const gdouble x = lnzeta - data->mu;
-  const gdouble exp_arg = -(x * x / (2.0 * D_SZ * D_SZ));
-  const gdouble xi_mean = _nc_cluster_mass_benson_xi_mean (zeta);
-  const gdouble plnzeta = exp (exp_arg) / (zeta * M_SQRT2 * M_SQRTPI * D_SZ);
-  const gdouble xi_zeta_int = _nc_cluster_mass_benson_int_xi_cut_inf (xi_mean, msz->signif_obs_min); 
-
-  //printf ("nhaca % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g\n", zeta, exp_arg, xi_mean, plnzeta, xi_zeta_int);
-
+  integrand_data *data      = (integrand_data *) userdata;
+  NcClusterMassBenson *msz  = data->msz;
+  const gdouble lnzeta      = log (zeta);
+  const gdouble x           = lnzeta - data->mu;
+  const gdouble exp_arg     = -(x * x / (2.0 * D_SZ * D_SZ));
+  const gdouble xi_mean     = _nc_cluster_mass_benson_xi_mean (zeta);
+  const gdouble plnzeta     = exp (exp_arg) / (zeta * M_SQRT2 * M_SQRTPI * D_SZ);
+  const gdouble xi_zeta_int = _nc_cluster_mass_benson_int_xi_cut_inf (xi_mean, msz->signif_obs_min);
+  
+  /*printf ("nhaca % 20.15g % 20.15g % 20.15g % 20.15g % 20.15g\n", zeta, exp_arg, xi_mean, plnzeta, xi_zeta_int); */
+  
   return plnzeta * xi_zeta_int;
 }
 
@@ -341,45 +345,47 @@ _nc_cluster_mass_benson_significance_m_p (NcClusterMass *clusterm, NcHICosmo *mo
   integrand_data data;
   NcClusterMassBenson *msz = NC_CLUSTER_MASS_BENSON (clusterm);
   gdouble P, err;
-  const gdouble E0 = nc_hicosmo_E (model, msz->z0); 
-  const gdouble E = nc_hicosmo_E (model, z);
+  const gdouble E0 = nc_hicosmo_E (model, msz->z0);
+  const gdouble E  = nc_hicosmo_E (model, z);
   gsl_function F;
   gsl_integration_workspace **w = ncm_integral_get_workspace ();
-
-  data.msz = msz;
-  data.model = model;
-  data.lnM = lnM;
-  data.z = z;
-  data.xi = xi;
+  
+  data.msz       = msz;
+  data.model     = model;
+  data.lnM       = lnM;
+  data.z         = z;
+  data.xi        = xi;
   data.xi_params = xi_params;
-
-  data.lnA = log (A_SZ);
-  data.lnM0 = log (msz->M0);
+  
+  data.lnA    = log (A_SZ);
+  data.lnM0   = log (msz->M0);
   data.lnE_E0 = log (E / E0);
-  data.mu = B_SZ * (lnM - data.lnM0) + C_SZ * data.lnE_E0 + data.lnA;
-  data.D2_2 = 2.0 * D_SZ * D_SZ;
-
+  data.mu     = B_SZ * (lnM - data.lnM0) + C_SZ * data.lnE_E0 + data.lnA;
+  data.D2_2   = 2.0 * D_SZ * D_SZ;
+  
   F.function = &_nc_cluster_mass_benson_significance_m_p_integrand;
-  F.params = &data;
-
+  F.params   = &data;
+  
   {
     gdouble Pi, a, b;
-//    a = 0.25;
-//    b = 1.0;
+    
+/*    a = 0.25; */
+/*    b = 1.0; */
     a = 0.0;
     b = 1.0;
     gsl_integration_qag (&F, a, b, 0.0, NCM_DEFAULT_PRECISION, NCM_INTEGRAL_PARTITION, 6, *w, &Pi, &err);
     P = Pi;
-//    b = 2.0;
+/*    b = 2.0; */
     b = 1.0;
+    
     do {
-      a = b;
+      a  = b;
       b += xi[0];
-      gsl_integration_qag (&F, a, b, P * NCM_DEFAULT_PRECISION, NCM_DEFAULT_PRECISION, NCM_INTEGRAL_PARTITION, 6, *w, &Pi, &err); 
+      gsl_integration_qag (&F, a, b, P * NCM_DEFAULT_PRECISION, NCM_DEFAULT_PRECISION, NCM_INTEGRAL_PARTITION, 6, *w, &Pi, &err);
       P += Pi;
-    } while (fabs(Pi/P) > NCM_DEFAULT_PRECISION);
+    } while (fabs (Pi / P) > NCM_DEFAULT_PRECISION);
   }
-
+  
   ncm_memory_pool_return (w);
   
   return P;
@@ -391,48 +397,50 @@ _nc_cluster_mass_benson_intp (NcClusterMass *clusterm, NcHICosmo *model, gdouble
   integrand_data data;
   NcClusterMassBenson *msz = NC_CLUSTER_MASS_BENSON (clusterm);
   gdouble P, err;
-  const gdouble E0 = nc_hicosmo_E (model, msz->z0); 
-  const gdouble E = nc_hicosmo_E (model, z);
+  const gdouble E0 = nc_hicosmo_E (model, msz->z0);
+  const gdouble E  = nc_hicosmo_E (model, z);
   gsl_function F;
   gsl_integration_workspace **w = ncm_integral_get_workspace ();
-
-  data.msz = msz;
+  
+  data.msz   = msz;
   data.model = model;
-  data.lnM = lnM;
-  data.z = z;
-
-  data.lnA = log (A_SZ);
-  data.lnM0 = log (msz->M0);
+  data.lnM   = lnM;
+  data.z     = z;
+  
+  data.lnA    = log (A_SZ);
+  data.lnM0   = log (msz->M0);
   data.lnE_E0 = log (E / E0);
-  data.mu = B_SZ * (lnM - data.lnM0) + C_SZ * data.lnE_E0 + data.lnA;
-  data.D2_2 = 2.0 * D_SZ * D_SZ;
-
+  data.mu     = B_SZ * (lnM - data.lnM0) + C_SZ * data.lnE_E0 + data.lnA;
+  data.D2_2   = 2.0 * D_SZ * D_SZ;
+  
   F.function = &_nc_cluster_mass_benson_significance_m_intp_integrand;
-  F.params = &data;
+  F.params   = &data;
   
   {
     gdouble Pi, a, b;
-//    a = 0.25;
-//    b = 1.0;
+    
+/*    a = 0.25; */
+/*    b = 1.0; */
     a = 0.0;
     b = 1.0;
     gsl_integration_qag (&F, a, b, 0.0, NCM_DEFAULT_PRECISION, NCM_INTEGRAL_PARTITION, 6, *w, &Pi, &err);
     P = Pi;
-//    b = 2.0;
+/*    b = 2.0; */
     b = 1.0;
-//printf ("int_p[0,1] % 8.5g % 8.5g : % 8.5g % 8.5g\n", exp (lnM), z, P, data.mu);
+    
+/*printf ("int_p[0,1] % 8.5g % 8.5g : % 8.5g % 8.5g\n", exp (lnM), z, P, data.mu); */
     do {
-      a = b;
+      a  = b;
       b += msz->signif_obs_max;
-      gsl_integration_qag (&F, a, b, P * NCM_DEFAULT_PRECISION, NCM_DEFAULT_PRECISION, NCM_INTEGRAL_PARTITION, 6, *w, &Pi, &err); 
+      gsl_integration_qag (&F, a, b, P * NCM_DEFAULT_PRECISION, NCM_DEFAULT_PRECISION, NCM_INTEGRAL_PARTITION, 6, *w, &Pi, &err);
       P += Pi;
-    } while (fabs(Pi/P) > NCM_DEFAULT_PRECISION);
+    } while (fabs (Pi / P) > NCM_DEFAULT_PRECISION);
   }
-
+  
   ncm_memory_pool_return (w);
-
-//printf ("int_p[2,-] % 8.5g % 8.5g : % 8.5g % 8.5g\n", exp (lnM), z, P, data.mu);
-
+  
+/*printf ("int_p[2,-] % 8.5g % 8.5g : % 8.5g % 8.5g\n", exp (lnM), z, P, data.mu); */
+  
   
   return P;
 }
@@ -443,29 +451,35 @@ _nc_cluster_mass_benson_resample (NcClusterMass *clusterm, NcHICosmo *model, gdo
   NcClusterMassBenson *msz = NC_CLUSTER_MASS_BENSON (clusterm);
   gdouble lnzeta, lnzeta_obs, zeta_obs, xi_mean;
   const gdouble E0 = nc_hicosmo_E (model, msz->z0);
-  const gdouble E = nc_hicosmo_E (model, z);
-
+  const gdouble E  = nc_hicosmo_E (model, z);
+  
   NCM_UNUSED (xi_params);
   
   lnzeta = B_SZ * (lnM - log (msz->M0)) + C_SZ * log (E / E0) + log (A_SZ);
- 
+  
   {
     gboolean ret;
+    
     ncm_rng_lock (rng);
     lnzeta_obs = lnzeta + gsl_ran_gaussian (rng->r, D_SZ);
-    zeta_obs = exp (lnzeta_obs);
-    if (zeta_obs > 1.0 && zeta_obs < 2.0)
+    zeta_obs   = exp (lnzeta_obs);
+    
+    if ((zeta_obs > 1.0) && (zeta_obs < 2.0))
+    {
       ret = FALSE;
+    }
     else
     {
       xi_mean = _nc_cluster_mass_benson_xi_mean (zeta_obs);
-      xi[0] = xi_mean + gsl_ran_gaussian (rng->r, 1.0);
-
-      //printf("M = %e z = %.5g zeta = %.5g xi = %.5g xiobs = %.5g | xiobs_min = %.5g\n", exp(lnM), z, zeta_obs, xi_mean, xi[0], msz->signif_obs_min);
-
+      xi[0]   = xi_mean + gsl_ran_gaussian (rng->r, 1.0);
+      
+      /*printf("M = %e z = %.5g zeta = %.5g xi = %.5g xiobs = %.5g | xiobs_min = %.5g\n", exp(lnM), z, zeta_obs, xi_mean, xi[0], msz->signif_obs_min); */
+      
       ret = (xi[0] >= msz->signif_obs_min);
     }
+    
     ncm_rng_unlock (rng);
+    
     return ret;
   }
 }
@@ -484,12 +498,12 @@ static gdouble
 _zeta_to_mass (NcClusterMass *clusterm, NcHICosmo *model, gdouble z, gdouble zeta)
 {
   NcClusterMassBenson *msz = NC_CLUSTER_MASS_BENSON (clusterm);
-  const gdouble E0 = nc_hicosmo_E (model, msz->z0); 
-  const gdouble E = nc_hicosmo_E (model, z);
-  const gdouble lnzeta = log (zeta);
-  const gdouble lnM = log (msz->M0) + (lnzeta - log (A_SZ) - C_SZ * log (E / E0)) / B_SZ;
-
-  //printf("z= %.10g xi = %.10g lnM = %.10g\n", z, xi, lnM);
+  const gdouble E0         = nc_hicosmo_E (model, msz->z0);
+  const gdouble E          = nc_hicosmo_E (model, z);
+  const gdouble lnzeta     = log (zeta);
+  const gdouble lnM        = log (msz->M0) + (lnzeta - log (A_SZ) - C_SZ * log (E / E0)) / B_SZ;
+  
+  /*printf("z= %.10g xi = %.10g lnM = %.10g\n", z, xi, lnM); */
   return lnM;
 }
 
@@ -497,19 +511,19 @@ static void
 _nc_cluster_mass_benson_p_limits (NcClusterMass *clusterm, NcHICosmo *model, const gdouble *xi, const gdouble *xi_params, gdouble *lnM_lower, gdouble *lnM_upper)
 {
   NcClusterMassBenson *msz = NC_CLUSTER_MASS_BENSON (clusterm);
-  const gdouble xil = GSL_MAX (xi[0] - 7.0, msz->signif_obs_min);
-  const gdouble zetal = _significance_to_zeta (clusterm, model, 2.0, xil) - 7.0 * D_SZ;
-  const gdouble lnMl = GSL_MAX (_zeta_to_mass (clusterm, model, 2.0, zetal), log (NC_CLUSTER_MASS_BENSON_M_LOWER_BOUND));
-
-  const gdouble xiu = xi[0] + 7.0;
+  const gdouble xil        = GSL_MAX (xi[0] - 7.0, msz->signif_obs_min);
+  const gdouble zetal      = _significance_to_zeta (clusterm, model, 2.0, xil) - 7.0 * D_SZ;
+  const gdouble lnMl       = GSL_MAX (_zeta_to_mass (clusterm, model, 2.0, zetal), log (NC_CLUSTER_MASS_BENSON_M_LOWER_BOUND));
+  
+  const gdouble xiu   = xi[0] + 7.0;
   const gdouble zetau = _significance_to_zeta (clusterm, model, 0.0, xiu) + 7.0 * D_SZ;
-  const gdouble lnMu = _zeta_to_mass (clusterm, model, 0.0, zetau);
-
+  const gdouble lnMu  = _zeta_to_mass (clusterm, model, 0.0, zetau);
+  
   NCM_UNUSED (xi_params);
   
   *lnM_lower = lnMl;
   *lnM_upper = lnMu;
-
+  
   return;
 }
 
@@ -517,18 +531,18 @@ static void
 _nc_cluster_mass_benson_n_limits (NcClusterMass *clusterm, NcHICosmo *model, gdouble *lnM_lower, gdouble *lnM_upper)
 {
   NcClusterMassBenson *msz = NC_CLUSTER_MASS_BENSON (clusterm);
-
-  const gdouble xil = msz->signif_obs_min;
+  
+  const gdouble xil   = msz->signif_obs_min;
   const gdouble zetal = _significance_to_zeta (clusterm, model, 2.0, xil) - 7.0 * D_SZ;
-  const gdouble lnMl = GSL_MAX (_zeta_to_mass (clusterm, model, 2.0, zetal), log (NC_CLUSTER_MASS_BENSON_M_LOWER_BOUND));
-
-  const gdouble xiu = msz->signif_obs_max;
+  const gdouble lnMl  = GSL_MAX (_zeta_to_mass (clusterm, model, 2.0, zetal), log (NC_CLUSTER_MASS_BENSON_M_LOWER_BOUND));
+  
+  const gdouble xiu   = msz->signif_obs_max;
   const gdouble zetau = _significance_to_zeta (clusterm, model, 0.0, xiu) + 7.0 * D_SZ;
-  const gdouble lnMu = _zeta_to_mass (clusterm, model, 0.0, zetau);
-
+  const gdouble lnMu  = _zeta_to_mass (clusterm, model, 0.0, zetau);
+  
   *lnM_lower = lnMl;
   *lnM_upper = lnMu;
-
-  return;  
+  
+  return;
 }
 
