@@ -3,11 +3,11 @@
  *
  *  Fri February 24 21:18:21 2012
  *  Copyright  2012  Sandro Dias Pinto Vitenti
- *  <sandro@isoftware.com.br>
+ *  <vitenti@uel.br>
  ****************************************************************************/
 /*
  * numcosmo
- * Copyright (C) Sandro Dias Pinto Vitenti 2012 <sandro@isoftware.com.br>
+ * Copyright (C) Sandro Dias Pinto Vitenti 2012 <vitenti@uel.br>
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
@@ -221,6 +221,7 @@ void ncm_model_param_set_lower_bound (NcmModel *model, guint n, const gdouble lb
 void ncm_model_param_set_upper_bound (NcmModel *model, guint n, const gdouble ub);
 void ncm_model_param_set_abstol (NcmModel *model, guint n, const gdouble abstol);
 void ncm_model_param_set_ftype (NcmModel *model, guint n, const NcmParamType ptype);
+void ncm_model_params_set_default_ftype (NcmModel *model);
 
 void ncm_model_reparam_df (NcmModel *model, NcmVector *fv, NcmVector *v);
 void ncm_model_reparam_J (NcmModel *model, NcmMatrix *fJ, NcmMatrix *J);
