@@ -3,11 +3,11 @@
  *
  *  Tue Feb  2 22:16:05 2010
  *  Copyright  2010  Sandro Dias Pinto Vitenti
- *  <sandro@isoftware.com.br>
+ *  <vitenti@uel.br>
  ****************************************************************************/
 /*
  * numcosmo
- * Copyright (C) Sandro Dias Pinto Vitenti 2012 <sandro@isoftware.com.br>
+ * Copyright (C) Sandro Dias Pinto Vitenti 2012 <vitenti@uel.br>
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
@@ -37,7 +37,9 @@
 G_BEGIN_DECLS
 
 void ncm_mpsf_sin_int_mpfr (mpq_t q, mpfr_ptr res, mp_rnd_t rnd);
+gdouble ncm_sf_sin_int (gdouble x);
 
 G_END_DECLS
 
 #endif /* _NCM_MPSF_TRIG_INT_H */
+
