@@ -3,12 +3,12 @@
  *
  *  Sun Aug  1 17:17:08 2010
  *  Copyright  2010  Mariana Penna Lima & Sandro Dias Pinto Vitenti
- *  <pennalima@gmail.com>, <sandro@isoftware.com.br>
+ *  <pennalima@gmail.com>, <vitenti@uel.br>
  ****************************************************************************/
 
 /*
  * numcosmo
- * Copyright (C) Mariana Penna Lima & Sandro Dias Pinto Vitenti 2012 <pennalima@gmail.com>, <sandro@isoftware.com.br>
+ * Copyright (C) Mariana Penna Lima & Sandro Dias Pinto Vitenti 2012 <pennalima@gmail.com>, <vitenti@uel.br>
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
@@ -61,6 +61,10 @@ ncm_spline2d_init (NcmSpline2d *s2d)
 {
   s2d->xv        = NULL;
   s2d->yv        = NULL;
+  s2d->x_interv  = 0;
+  s2d->y_interv  = 0;
+  s2d->x_data    = NULL;
+  s2d->y_data    = NULL;
   s2d->zm        = NULL;
   s2d->s         = NULL;
   s2d->empty     = TRUE;
@@ -78,9 +82,9 @@ static void
 _ncm_spline2d_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
   NcmSpline2d *s2d = NCM_SPLINE2D (object);
-  
+
   g_return_if_fail (NCM_IS_SPLINE2D (object));
-  
+
   switch (prop_id)
   {
     case PROP_SPLINE:
@@ -101,7 +105,7 @@ _ncm_spline2d_set_property (GObject *object, guint prop_id, const GValue *value,
     case PROP_INIT:
     {
       s2d->to_init = g_value_get_boolean (value);
-      
+
       if (s2d->to_init && (s2d->xv != NULL) && (s2d->yv != NULL) && (s2d->zm != NULL))
         ncm_spline2d_prepare (s2d);
 
@@ -110,9 +114,9 @@ _ncm_spline2d_set_property (GObject *object, guint prop_id, const GValue *value,
     case PROP_USE_ACC:
       ncm_spline2d_use_acc (s2d, g_value_get_boolean (value));
       break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
+    default:                                                      /* LCOV_EXCL_LINE */
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec); /* LCOV_EXCL_LINE */
+      break;                                                      /* LCOV_EXCL_LINE */
   }
 }
 
@@ -120,9 +124,9 @@ static void
 _ncm_spline2d_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
   NcmSpline2d *s2d = NCM_SPLINE2D (object);
-  
+
   g_return_if_fail (NCM_IS_SPLINE2D (object));
-  
+
   switch (prop_id)
   {
     case PROP_SPLINE:
@@ -143,9 +147,9 @@ _ncm_spline2d_get_property (GObject *object, guint prop_id, GValue *value, GPara
     case PROP_USE_ACC:
       g_value_set_boolean (value, s2d->use_acc);
       break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
+    default:                                                      /* LCOV_EXCL_LINE */
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec); /* LCOV_EXCL_LINE */
+      break;                                                      /* LCOV_EXCL_LINE */
   }
 }
 
@@ -153,14 +157,14 @@ static void
 ncm_spline2d_dispose (GObject *object)
 {
   NcmSpline2d *s2d = NCM_SPLINE2D (object);
-  
+
   ncm_vector_clear (&s2d->xv);
   ncm_vector_clear (&s2d->yv);
   ncm_matrix_clear (&s2d->zm);
   ncm_spline_clear (&s2d->s);
-  
+
   s2d->empty = TRUE;
-  
+
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_spline2d_parent_class)->dispose (object);
 }
@@ -169,10 +173,10 @@ static void
 ncm_spline2d_finalize (GObject *object)
 {
   NcmSpline2d *s2d = NCM_SPLINE2D (object);
-  
+
   g_clear_pointer (&s2d->acc_x, gsl_interp_accel_free);
   g_clear_pointer (&s2d->acc_y, gsl_interp_accel_free);
-  
+
   /* Chain up : end */
   G_OBJECT_CLASS (ncm_spline2d_parent_class)->finalize (object);
 }
@@ -181,12 +185,12 @@ static void
 ncm_spline2d_class_init (NcmSpline2dClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  
+
   object_class->set_property = &_ncm_spline2d_set_property;
   object_class->get_property = &_ncm_spline2d_get_property;
   object_class->dispose      = &ncm_spline2d_dispose;
   object_class->finalize     = &ncm_spline2d_finalize;
-  
+
   klass->copy_empty    = NULL;
   klass->reset         = NULL;
   klass->prepare       = NULL;
@@ -201,7 +205,8 @@ ncm_spline2d_class_init (NcmSpline2dClass *klass)
   klass->int_dxdy      = NULL;
   klass->int_dx_spline = NULL;
   klass->int_dy_spline = NULL;
-  
+  klass->eval_vec_y    = NULL;
+
   /**
    * NcmSpline2d:spline:
    *
@@ -214,7 +219,7 @@ ncm_spline2d_class_init (NcmSpline2dClass *klass)
                                                         "Spline",
                                                         NCM_TYPE_SPLINE,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  
+
   /**
    * NcmSpline2d:xv:
    *
@@ -227,7 +232,7 @@ ncm_spline2d_class_init (NcmSpline2dClass *klass)
                                                         "x vector",
                                                         NCM_TYPE_VECTOR,
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  
+
   /**
    * NcmSpline2d:yv:
    *
@@ -240,7 +245,7 @@ ncm_spline2d_class_init (NcmSpline2dClass *klass)
                                                         "y vector",
                                                         NCM_TYPE_VECTOR,
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  
+
   /**
    * NcmSpline2d:zm:
    *
@@ -253,7 +258,7 @@ ncm_spline2d_class_init (NcmSpline2dClass *klass)
                                                         "z matrix",
                                                         NCM_TYPE_MATRIX,
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  
+
   /**
    * NcmSpline2d:init:
    *
@@ -266,7 +271,7 @@ ncm_spline2d_class_init (NcmSpline2dClass *klass)
                                                          "init",
                                                          FALSE,
                                                          G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB));
-  
+
   /**
    * NcmSpline2d:use_acc:
    *
@@ -290,26 +295,30 @@ _ncm_spline2d_makeup (NcmSpline2d *s2d)
     g_assert_cmpuint (ncm_vector_len (s2d->yv), ==, ncm_matrix_col_len (s2d->zm));
     g_assert_cmpuint (ncm_vector_len (s2d->xv), >=, ncm_spline2d_min_size (s2d));
     g_assert_cmpuint (ncm_vector_len (s2d->yv), >=, ncm_spline2d_min_size (s2d));
-    
-    s2d->empty = FALSE;
-    
+
+    s2d->empty    = FALSE;
+    s2d->x_interv = ncm_vector_len (s2d->xv) - 1;
+    s2d->y_interv = ncm_vector_len (s2d->yv) - 1;
+    s2d->x_data   = ncm_vector_data (s2d->xv);
+    s2d->y_data   = ncm_vector_data (s2d->yv);
+
     if ((ncm_vector_stride (s2d->xv) == 1) && (ncm_vector_stride (s2d->yv) == 1))
       s2d->no_stride = TRUE;
     else
       s2d->no_stride = FALSE;
-    
+
     if (s2d->use_acc && !s2d->no_stride)
     {
       g_warning ("_ncm_spline2d_makeup: use-acc true but strided knots vectors, disabling use-add.");
       s2d->use_acc = FALSE;
     }
-    
+
     NCM_SPLINE2D_GET_CLASS (s2d)->reset (s2d);
-    
+
     if (s2d->to_init)
       ncm_spline2d_prepare (s2d);
   }
-  
+
   return;
 }
 
@@ -327,15 +336,23 @@ _ncm_spline2d_makeup (NcmSpline2d *s2d)
 void
 ncm_spline2d_set (NcmSpline2d *s2d, NcmVector *xv, NcmVector *yv, NcmMatrix *zm, gboolean init)
 {
+  NcmVector *old_xv = s2d->xv;
+  NcmVector *old_yv = s2d->yv;
+  NcmMatrix *old_zm = s2d->zm;
+
   g_assert ((xv != NULL) && (yv != NULL) && (zm != NULL));
-  
+
   s2d->xv = ncm_vector_ref (xv);
   s2d->yv = ncm_vector_ref (yv);
   s2d->zm = ncm_matrix_ref (zm);
-  
+
+  ncm_vector_clear (&old_xv);
+  ncm_vector_clear (&old_yv);
+  ncm_matrix_clear (&old_zm);
+
   s2d->to_init = init;
   _ncm_spline2d_makeup (s2d);
-  
+
   return;
 }
 
@@ -367,16 +384,20 @@ NcmSpline2d *
 ncm_spline2d_copy (NcmSpline2d *s2d)
 {
   NcmSpline2d *new_s2d = ncm_spline2d_copy_empty (s2d);
-  
-  if (!s2d->empty)
-    ncm_spline2d_set (new_s2d,
-                      ncm_vector_dup (s2d->xv),
-                      ncm_vector_dup (s2d->yv),
-                      ncm_matrix_dup (s2d->zm),
-                      s2d->init
-                     );
 
-  
+  if (!s2d->empty)
+  {
+    NcmVector *xv = ncm_vector_dup (s2d->xv);
+    NcmVector *yv = ncm_vector_dup (s2d->yv);
+    NcmMatrix *zm = ncm_matrix_dup (s2d->zm);
+
+    ncm_spline2d_set (new_s2d, xv, yv, zm, s2d->init);
+
+    ncm_vector_free (xv);
+    ncm_vector_free (yv);
+    ncm_matrix_free (zm);
+  }
+
   return new_s2d;
 }
 
@@ -398,9 +419,9 @@ NcmSpline2d *
 ncm_spline2d_new (const NcmSpline2d *s2d, NcmVector *xv, NcmVector *yv, NcmMatrix *zm, gboolean init)
 {
   NcmSpline2d *s2d_new = ncm_spline2d_copy_empty (s2d);
-  
+
   ncm_spline2d_set (s2d_new, xv, yv, zm, init);
-  
+
   return s2d_new;
 }
 
@@ -420,7 +441,7 @@ ncm_spline2d_min_size (NcmSpline2d *s2d)
  * ncm_spline2d_prepare:
  * @s2d: a #NcmSpline2d
  *
- * This function prepares the bidimensional spline @s2d such that one can evaluate it (#ncm_spline2d_eval),
+ * This function prepares the bi-dimensional spline @s2d such that one can evaluate it (#ncm_spline2d_eval),
  * as well as to compute its integration in x, y or both directions.
  */
 void
@@ -429,6 +450,20 @@ ncm_spline2d_prepare (NcmSpline2d *s2d)
   NCM_SPLINE2D_GET_CLASS (s2d)->prepare (s2d);
 }
 
+/**
+ * ncm_spline2d_ref:
+ * @s2d: a #NcmSpline2d
+ *
+ * Atomically increases the reference count of @s2d by one.
+ *
+ * Returns: (transfer full): the same object @s2d.
+ */
+NcmSpline2d *
+ncm_spline2d_ref (NcmSpline2d *s2d)
+{
+  return g_object_ref (s2d);
+}
+
 /**
  * ncm_spline2d_free:
  * @s2d: a #NcmSpline2d
@@ -469,7 +504,7 @@ void
 ncm_spline2d_use_acc (NcmSpline2d *s2d, gboolean use_acc)
 {
   s2d->use_acc = use_acc;
-  
+
   if ((s2d->xv != NULL) && (s2d->yv != NULL) && (s2d->zm != NULL))
   {
     if (s2d->use_acc && !s2d->no_stride)
@@ -496,8 +531,8 @@ gdouble
 ncm_spline2d_integ_dx (NcmSpline2d *s2d, gdouble xl, gdouble xu, gdouble y)
 {
   if (!s2d->init)
-    ncm_spline2d_prepare (s2d);
-  
+    ncm_spline2d_prepare (s2d);  /* LCOV_EXCL_LINE */
+
   return NCM_SPLINE2D_GET_CLASS (s2d)->int_dx (s2d, xl, xu, y);
 }
 
@@ -517,8 +552,8 @@ gdouble
 ncm_spline2d_integ_dy (NcmSpline2d *s2d, gdouble x, gdouble yl, gdouble yu)
 {
   if (!s2d->init)
-    ncm_spline2d_prepare (s2d);
-  
+    ncm_spline2d_prepare (s2d);  /* LCOV_EXCL_LINE */
+
   return NCM_SPLINE2D_GET_CLASS (s2d)->int_dy (s2d, x, yl, yu);
 }
 
@@ -539,8 +574,8 @@ gdouble
 ncm_spline2d_integ_dxdy (NcmSpline2d *s2d, gdouble xl, gdouble xu, gdouble yl, gdouble yu)
 {
   if (!s2d->init)
-    ncm_spline2d_prepare (s2d);
-  
+    ncm_spline2d_prepare (s2d);  /* LCOV_EXCL_LINE */
+
   return NCM_SPLINE2D_GET_CLASS (s2d)->int_dxdy (s2d, xl, xu, yl, yu);
 }
 
@@ -559,10 +594,10 @@ NcmSpline *
 ncm_spline2d_integ_dx_spline (NcmSpline2d *s2d, gdouble xl, gdouble xu)
 {
   g_assert (!s2d->empty);
-  
+
   if (!s2d->init)
-    ncm_spline2d_prepare (s2d);
-  
+    ncm_spline2d_prepare (s2d);  /* LCOV_EXCL_LINE */
+
   return ncm_spline_copy (NCM_SPLINE2D_GET_CLASS (s2d)->int_dx_spline (s2d, xl, xu));
 }
 
@@ -581,8 +616,8 @@ NcmSpline *
 ncm_spline2d_integ_dy_spline (NcmSpline2d *s2d, gdouble yl, gdouble yu)
 {
   if (!s2d->init)
-    ncm_spline2d_prepare (s2d);
-  
+    ncm_spline2d_prepare (s2d);  /* LCOV_EXCL_LINE */
+
   return ncm_spline_copy (NCM_SPLINE2D_GET_CLASS (s2d)->int_dy_spline (s2d, yl, yu));
 }
 
@@ -602,8 +637,8 @@ gdouble
 ncm_spline2d_integ_dx_spline_val (NcmSpline2d *s2d, gdouble xl, gdouble xu, gdouble y)
 {
   if (!s2d->init)
-    ncm_spline2d_prepare (s2d);
-  
+    ncm_spline2d_prepare (s2d);  /* LCOV_EXCL_LINE */
+
   return ncm_spline_eval (NCM_SPLINE2D_GET_CLASS (s2d)->int_dx_spline (s2d, xl, xu), y);
 }
 
@@ -623,8 +658,8 @@ gdouble
 ncm_spline2d_integ_dy_spline_val (NcmSpline2d *s2d, gdouble x, gdouble yl, gdouble yu)
 {
   if (!s2d->init)
-    ncm_spline2d_prepare (s2d);
-  
+    ncm_spline2d_prepare (s2d);  /* LCOV_EXCL_LINE */
+
   return ncm_spline_eval (NCM_SPLINE2D_GET_CLASS (s2d)->int_dy_spline (s2d, yl, yu), x);
 }
 
@@ -645,8 +680,8 @@ gdouble
 ncm_spline2d_integ_dxdy_spline_x (NcmSpline2d *s2d, gdouble xl, gdouble xu, gdouble yl, gdouble yu)
 {
   if (!s2d->init)
-    ncm_spline2d_prepare (s2d);
-  
+    ncm_spline2d_prepare (s2d);  /* LCOV_EXCL_LINE */
+
   return ncm_spline_eval_integ (NCM_SPLINE2D_GET_CLASS (s2d)->int_dx_spline (s2d, xl, xu), yl, yu);
 }
 
@@ -667,8 +702,8 @@ gdouble
 ncm_spline2d_integ_dxdy_spline_y (NcmSpline2d *s2d, gdouble xl, gdouble xu, gdouble yl, gdouble yu)
 {
   if (!s2d->init)
-    ncm_spline2d_prepare (s2d);
-  
+    ncm_spline2d_prepare (s2d);  /* LCOV_EXCL_LINE */
+
   return ncm_spline_eval_integ (NCM_SPLINE2D_GET_CLASS (s2d)->int_dy_spline (s2d, yl, yu), xl, xu);
 }
 
@@ -735,6 +770,18 @@ ncm_spline2d_integ_dxdy_spline_y (NcmSpline2d *s2d, gdouble xl, gdouble xu, gdou
  * over the entire valid ranges of x and y coordinates.
  */
 
+/**
+ * ncm_spline2d_eval_vec_y:
+ * @s2d: a #NcmSpline2d
+ * @x: x-coordinate value
+ * @y: a #NcmVector
+ * @order: (element-type size_t) (allow-none): FIXME
+ * @res: (element-type gdouble): a #NcmVector
+ *
+ * FIXME
+ *
+ */
+
 /*******************************************************************************
  * Autoknots
  *******************************************************************************/
@@ -769,25 +816,19 @@ ncm_spline2d_set_function (NcmSpline2d *s2d, NcmSplineFuncType ftype, gsl_functi
 {
   NcmSpline *s_x = ncm_spline_copy_empty (s2d->s);
   NcmSpline *s_y = ncm_spline_copy_empty (s2d->s);
-  
+
   ncm_spline_set_func (s_x, ftype, Fx, xl, xu, 0, rel_err);
   ncm_spline_set_func (s_y, ftype, Fy, yl, yu, 0, rel_err);
-  
-/*
- *  printf ("x % 22.15g % 22.15g %u | y % 22.15g % 22.15g %u\n", xl, xu, ncm_vector_len (s_y->xv), yl, yu, ncm_vector_len (s_x->xv));
- *  ncm_vector_log_vals (s_y->xv, "XV", "% 22.15g", TRUE);
- *  ncm_vector_log_vals (s_x->xv, "XV", "% 22.15g", TRUE);
- */
   {
     NcmMatrix *s_z = ncm_matrix_new (ncm_vector_len (s_y->xv), ncm_vector_len (s_x->xv));
-    
+
     ncm_spline2d_set (s2d, s_x->xv, s_y->xv, s_z, FALSE);
     ncm_matrix_free (s_z);
   }
-  
+
   ncm_spline_free (s_x);
   ncm_spline_free (s_y);
-  
+
   return;
 }
 
