@@ -3,12 +3,12 @@
  *
  *  Tue Jul  8 15:05:41 2008
  *  Copyright  2008  Sandro Dias Pinto Vitenti
- *  <sandro@isoftware.com.br>
+ *  <vitenti@uel.br>
  ****************************************************************************/
 
 /*
  * numcosmo
- * Copyright (C) Sandro Dias Pinto Vitenti 2012 <sandro@isoftware.com.br>
+ * Copyright (C) Sandro Dias Pinto Vitenti 2012 <vitenti@uel.br>
  * numcosmo is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
@@ -1187,6 +1187,41 @@ ncm_vector_log_vals_func (const NcmVector *cv, const gchar *prestr, const gchar
  * Returns: whether all components of @cv are finite.
  */
 
+/**
+ * ncm_vector_lt:
+ * @cv1: a constant #NcmVector
+ * @cv2: a constant #NcmVector
+ *
+ * Compare @cv1 and @cv2 and returns TRUE if all components
+ * of @cv1 are less than @cv2's.
+ *
+ * Returns: whether @cv1 < @cv2.
+ */
+/**
+ * ncm_vector_lteq:
+ * @cv1: a constant #NcmVector
+ * @cv2: a constant #NcmVector
+ *
+ * Compare @cv1 and @cv2 and returns TRUE if all components
+ * of @cv1 are less than or equal to @cv2's.
+ *
+ * Returns: whether @cv1 <= @cv2.
+ */
+/**
+ * ncm_vector_between:
+ * @cv: a constant #NcmVector
+ * @cv_lb: a constant #NcmVector
+ * @cv_ub: a constant #NcmVector
+ * @type: an int
+ *
+ * Check whether all components of @cv are between components
+ * of @cv_lb and @cv_ub.
+ * If type == 0, compare using @cv_lb <= @cv <  @cv_ub;
+ * If type == 1, compare using @cv_lb <  @cv <= @cv_ub;
+ *
+ * Returns: whether @cv is between @cv_lb and @cv_ub.
+ */
+
 /**
  * ncm_vector_get_absminmax:
  * @cv: a constant #NcmVector
@@ -1421,6 +1456,75 @@ ncm_vector_reciprocal (NcmVector *cv)
   }
 }
 
+/**
+ * ncm_vector_square:
+ * @cv: a #NcmVector
+ *
+ * Calculates the term-wise square of @cv.
+ *
+ */
+void
+ncm_vector_square (NcmVector *cv)
+{
+  const guint len = ncm_vector_len (cv);
+  guint i;
+
+  for (i = 0; i < len; i++)
+  {
+    const gdouble x_i = ncm_vector_get (cv, i);
+
+    ncm_vector_set (cv, i, x_i * x_i);
+  }
+}
+
+/**
+ * ncm_vector_sqrt:
+ * @cv: a #NcmVector
+ *
+ * Calculates the term-wise square-root of @cv.
+ *
+ */
+void
+ncm_vector_sqrt (NcmVector *cv)
+{
+  const guint len = ncm_vector_len (cv);
+  guint i;
+
+  for (i = 0; i < len; i++)
+  {
+    const gdouble x_i = ncm_vector_get (cv, i);
+
+    ncm_vector_set (cv, i, sqrt (x_i));
+  }
+}
+
+/**
+ * ncm_vector_hypot:
+ * @cv1: a #NcmVector
+ * @a: a constant gdouble
+ * @cv2: a constant #NcmVector
+ *
+ * Performs the operation: $x^i_1 = \sqrt{(x^i_1)^2+(\alpha x^i_2)^2}$
+ * where $x_1^i$ and $x_2^i$ are the components of @cv1 and @cv2
+ * respectively.
+ *
+ */
+void
+ncm_vector_hypot (NcmVector *cv1, const gdouble alpha, const NcmVector *cv2)
+{
+  const guint len = ncm_vector_len (cv1);
+  gint i;
+
+  g_assert_cmpuint (len, ==, ncm_vector_len (cv2));
+
+  for (i = 0; i < len; i++)
+  {
+    const gdouble x1_i = ncm_vector_get (cv1, i);
+    const gdouble x2_i = ncm_vector_get (cv2, i);
+    ncm_vector_set (cv1, i, hypot (x1_i, alpha * x2_i));
+  }
+}
+
 /**
  * ncm_vector_sum_cpts:
  * @cv: a constant #NcmVector
