@@ -4,7 +4,7 @@
  *
  *  Fri November 01 14:18:09 2019
  *  Copyright  2019  Sandro Dias Pinto Vitenti
- *  <sandro@isoftware.com.br>
+ *  <vitenti@uel.br>
  ****************************************************************************/
 /*
  * nc_hicosmo_qrbf.c
@@ -41,7 +41,7 @@
 #include "build_cfg.h"
 
 #include "model/nc_hicosmo_qrbf.h"
-#include "math/integral.h"
+#include "math/ncm_integrate.h"
 #include "math/ncm_memory_pool.h"
 
 
